#!/bin/sh

repo_strip_file_protocol() {
    echo "$1" | sed -s 's#^file:##'
}

repo_list_files() {
    local url
    url="$1"

    local dir

    case "$url" in
        file:/*)
            dir=$(repo_strip_file_protocol "$url")
            ls -1ct "$dir"
            return 0
            ;;
        ftp://*)
            curl "$url" --list-only
            return 0
            ;;
        *)
            error "Protocol is not implemented in repo_list_files() for $url"
            ;;
    esac
}

repo_get_canonic_url() {
    local url
    url="$1"

    local protocol

    case "$url" in
        /*)
            url="file://$url"
            ;;
    esac

    case "$url" in
        */dists/*)
            echo "$url" | sed -e 's#/dists/.*##'
            return 0
    esac

    echo "$url"
}

repo_get_codename() {
    local url
    url="$1"

    local canonic_url
    canonic_url="$2"

    local codename
    codename="$3"

    case "$url" in
        */dists/*)
            basename "$url"
            return 0
    esac

    test -z "$codename" || {
        echo "$codename"
        return 0
    }

    repo_list_files "$canonic_url"/dists/ | head -1 
}

get_release_file() {
    local url
    url="$1"/dists/"$2"

    local codename
    codename="$2"
    echo "$codename" >codename

    local filename
    filename=Release_$(repo_get_stamp "$url")

    repo_curl_newer "$url/Release" "$filename"

    awk '/^Version: / { print $2 }' "$filename" >version
    awk '
        /^SHA1:$/ {
            exit
        }
        /^ / {
            print $3
        }' "$filename" >cache_files
}

repo_get_components_from_cache_files() {
    local t
    t="$1"

    sed "/\/$t\$/!d; s#/.*##" cache_files | uniq
}

repo_create_source_list_from_cache_files() {
    local url
    url="$1"

    local codename
    codename="$2"

    local deb_components
    deb_components=$(repo_get_components_from_cache_files Packages.gz)

    local src_components
    src_components=$(repo_get_components_from_cache_files Sources.gz)

    test -z "$deb_components" || echo "deb [arch=$arch] $url $codename" $deb_components
    test -z "$src_components" || echo "deb-src $url $codename" $src_components
}

repo_create_source_list() {
    local url
    url="$1"

    local codename
    codename="$2"

    rm -f codename version

    test ! -f "$url" || {
        cp "$url" sources.list
        return 0
    }

    local canonic_url
    canonic_url=$(repo_get_canonic_url "$url")

    codename=$(repo_get_codename "$url" "$canonic_url" "$codename")

    get_release_file "$canonic_url" "$codename"
    repo_create_source_list_from_cache_files "$canonic_url" "$codename" >sources.list
}

repo_curl_newer() {
    local uri
    uri="$1"

    local file
    file="$2"
   
    local zflag
    test ! -e "$file" || zflag="-z '$file'"

    curl -s -o "$file" $zflag "$uri"
}

repo_get_stamp() {
    echo $@ | tr -cd '[:alnum:]._-'
}

