from texttable import Texttable
import html


class Markup:

    def __init__(self):
        self.sections = []
        self.title = None

    def add_h1(self, title):
        self.title = title

    def add_table(self, title, cols, rows):
        if not rows: return

        rows.sort()
        self.sections.append([title, cols, rows])

    def print(self):
        self.print_h1(self.title)

        self.print_h3('Contents')
        for section in self.sections:
            self.print_contents(section[0])

        for section in self.sections:
            self.print_h3(section[0])
            self.print_table(section[1], section[2])

    def get(markup):
        return Html() if markup == 'html' else Text()


class Text(Markup):

    def print_h1(self, title):
        print('# ' + title + ' #\n')

    def print_contents(self, title):
        print('*', title)

    def print_h3(self, title):
        print('\n### ' + title)

    def print_table(self, cols, rows):
        table = Texttable()
        table.set_deco(Texttable.HEADER)
        table.set_max_width(172)
        table.set_cols_dtype(len(cols) * 't')

        table.add_rows([cols] + rows)
        print(table.draw())


class Html(Markup):

    def print_h1(self, title):
        print('<h1>' + title + '</h1>')

    def print_contents(self, title):
        print('<a href="#' + title.replace(' ', '_') + '">' + title + '</a><br/>')

    def print_h3(self, title):
        print('<h3><a href="#Contents" name="' + title.replace(' ', '_') + '">' + title + '</a></h3>')

    def print_table(self, cols, rows):
        print('<table>')
        print('<tr><th>' + '</th><th>'.join(cols) + '</th></tr>')

        for r in rows:
            hr = [html.escape(x).replace('\n', '<br/>') for x in r]
            print('<tr><td>' + '</td><td>'.join(hr) + '</td></tr>')

        print('</table>')

