import argparse, sys
from markup import Markup


def parse_args():
    par = argparse.ArgumentParser(description = 'Draw a table from tab separated input')
    par.add_argument('--columns', dest='columns', action='store', help='comma separated column names')
    par.add_argument('--markup', dest='markup', default = 'text', action='store', help='html or text markup')
    args = par.parse_args()

    if not args.columns:
        sys.exit('Specify column names via --columns')

    return args.columns.split(','), Markup.get(args.markup)


def get_rows(NUM_COLS):
    DATA = sys.stdin.read().split('\t')
    NUM_DATA = len(DATA)
    NUM_TABS = NUM_COLS - 1

    rows = []
    pos = 0
    first = DATA[0]

    while pos + NUM_TABS < NUM_DATA:
        row = [first]
        row.extend(DATA[pos + 1: pos + NUM_TABS])

        last = DATA[pos + NUM_TABS]
        last, _, first = last.rpartition('\n')
        row.append(last)
        rows.append(row)
        pos += NUM_TABS

    return rows



COLS, MARKUP = parse_args()
ROWS = get_rows(len(COLS))

MARKUP.print_table(COLS, ROWS)

